# -*- coding: utf-8 -*-
try:
    from StringIO import StringIO
except:
    from io import StringIO
from django.conf import settings
from django.http import HttpResponseRedirect
from django.core.management import call_command
from django.template.base import TemplateDoesNotExist
from django.template.loader import BaseLoader
from django.test import SimpleTestCase
from tempfile import mkstemp
from statictemplate.management.commands.statictemplate import make_static, \
    InvalidResponseError

from . import settings as statictemplate_settings


class TestLoader(BaseLoader):
    is_usable = True
    templates = {
        'request': '{% extends "base" %}{% block content %}simple {{ request.GET.extra }}{% endblock %}',
        'simple': '{% extends "base" %}{% block content %}simple{% endblock %}',
        'base': '{% block head %}head{% endblock %}{% block content %}content{% endblock %}',
    }

    def load_template_source(self, template_name, template_dirs=None):
        found = self.templates.get(template_name, None)
        if not found:  # pragma: no cover
            raise TemplateDoesNotExist(template_name)
        return found, template_name


class MeddlingMiddleware(object):
    def process_request(self, request):
        return HttpResponseRedirect('/foobarbaz')


class StaticTemplateTests(SimpleTestCase):
    def setUp(self):
        settings.TEMPLATE_LOADERS = ['statictemplate.tests.TestLoader']

    def test_python_api(self):
        output = make_static('simple')
        self.assertEqual(output, 'headsimple')

    def test_call_command(self):
        sio = StringIO()
        call_command('statictemplate', 'simple', stdout=sio)
        self.assertEqual(sio.getvalue().strip(), 'headsimple')

    def test_request_command(self):
        sio = StringIO()
        call_command('statictemplate', 'request', stdout=sio,
                     extra_request='extra=extra_request&canonical=1')
        self.assertEqual(sio.getvalue().strip(), 'headsimple extra_request')

    def test_file_command(self):
        _, sio = mkstemp()
        call_command('statictemplate', 'simple', output=sio)
        with open(sio, 'r') as tmp:
            self.assertEqual(tmp.read().strip(), 'headsimple')

    def test_meddling_middleware(self):
        middleware = (
            'statictemplate.tests.MeddlingMiddleware',
        )
        settings.MIDDLEWARE_CLASSES = middleware
        output = make_static('simple')
        self.assertEqual(output, 'headsimple')
        self.assertEqual(settings.MIDDLEWARE_CLASSES, middleware)

    def test_no_ovveride_middleware(self):
        with self.settings(STATICTEMPLATE_OVERRIDE_MIDDLEWARE=False):
            middleware = (
                'statictemplate.tests.MeddlingMiddleware',
            )
            settings.MIDDLEWARE_CLASSES = middleware
            statictemplate_settings.OVERRIDE_MIDDLEWARE = False
            with self.assertRaises(InvalidResponseError):
                output = make_static('simple')
            statictemplate_settings.OVERRIDE_MIDDLEWARE = True


# -*- coding: utf-8 -*-
from contextlib import contextmanager
from optparse import make_option
try:
    import urlparse
except ImportError:  # NOQA
    from urllib import parse as urlparse
from django.conf import settings
try:
    from django.conf.urls.defaults import patterns, url, include
except ImportError:  # NOQA
    from django.conf.urls import patterns, url, include
from django.core.management.base import BaseCommand
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.test.client import Client
try:
    from django.utils.encoding import force_text
except ImportError:  # NOQA
    from django.utils.encoding import force_unicode as force_text
from django.utils.translation import get_language


class InvalidResponseError(Exception):
    pass


@contextmanager
def override_urlconf():
    has_old = hasattr(settings, 'ROOT_URLCONF')
    old = getattr(settings, 'ROOT_URLCONF', None)
    settings.ROOT_URLCONF = 'statictemplate.management.commands.statictemplate'
    yield
    if has_old:
        setattr(settings, 'ROOT_URLCONF', old)
    else:  # NOQA
        delattr(settings, 'ROOT_URLCONF')


@contextmanager
def override_middleware():
    from ...settings import OVERRIDE_MIDDLEWARE
    if OVERRIDE_MIDDLEWARE:
        has_old = hasattr(settings, 'MIDDLEWARE_CLASSES')
        old = getattr(settings, 'MIDDLEWARE_CLASSES', None)
        settings.MIDDLEWARE_CLASSES = (
            'django.middleware.common.CommonMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
        )
        yield
        if has_old:
            setattr(settings, 'MIDDLEWARE_CLASSES', old)
        else:  # NOQA
            delattr(settings, 'MIDDLEWARE_CLASSES')
    else:
        yield


def make_static(template, language=None, request=None):
    with override_urlconf():
        with override_middleware():
            client = Client()
            if not request:
                request = {}
            if language:
                client.cookies['django_language'] = language
            request.update({'template': template})
            response = client.get('/', request)
            if response.status_code != 200:
                raise InvalidResponseError(  # NOQA
                    'Response code was %d, expected 200' % response.status_code
                )
            return force_text(response.content)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--file', '-f',
                    action='store',
                    dest='output',
                    help='Output file'),
    )

    def handle(self, template, language=None, extra_request=None, **options):
        request = {}
        if not language:
            language = get_language()
        if extra_request:
            request.update(urlparse.parse_qs(extra_request,
                                             strict_parsing=True))
        output = make_static(template, language, request)
        if options.get('output', False):
            with open(options.get('output'), 'w') as output_file:
                output_file.write(output)
        else:
            self.stdout.write(output)


def render(request):
    template_name = request.GET['template']
    return render_to_response(template_name, RequestContext(request))


urlpatterns = patterns('',
    url('^$', render),
    url('^', include(settings.ROOT_URLCONF))
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
from statictemplate import __version__


INSTALL_REQUIRES = [
    'django>=1.2',
]

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Console',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Topic :: Software Development',
]

setup(
    name='django-statictemplate',
    version=__version__,
    description=('This project aims at providing a compromise between dynamic '
        'error pages for Django (that use template tags etc and therefore '
        'potentially error too) and having to write static error pages by hand.'),
    author='Jonas Obrist',
    author_email='ojiidotch@gmail.com',
    url='https://github.com/ojii/django-statictemplate',
    packages=[
        'statictemplate',
        'statictemplate.management',
        'statictemplate.management.commands'
    ],
    license='LICENSE',
    platforms=['OS Independent'],
    install_requires=INSTALL_REQUIRES,
)
